@@ -2,6 +2,8 @@
 
 trW <- function(W, m=100, p=50, type="mult") {
 # returns traces
+    timings <- list()
+    .ptime_start <- proc.time()
     n <- dim(W)[1]
     iW <- W
     tr <- numeric(m)
@@ -21,6 +23,9 @@ trW <- function(W, m=100, p=50, type="mult") {
         tr[1] <- 0.0
         tr[2] <- sum(t(W) * W)
     } else stop("unknown type")
+    timings[["make_traces"]] <- proc.time() - .ptime_start
+    attr(tr, "timings") <- do.call("rbind", timings)[, c(1, 3)]
+    attr(tr, "type") <- type
     tr
 }
 
@@ -57,9 +62,12 @@ impacts.stsls <- function(obj, ..., tr=NULL, R=NULL, listw=NULL,
 }
 
 intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
-    interval, type, tr, R, listw, tol, empirical, Q, icept, iicept, p) {
+    interval, type, tr, R, listw, tol, empirical, Q, icept, iicept, p,
+    mess=FALSE) {
     if (is.null(listw) && is.null(tr))
         stop("either tr or listw must be given")
+    timings <- list()
+    .ptime_start <- proc.time()
     if (is.null(listw)) {
 
         lagImpacts <- function(T, g, P) {
@@ -92,14 +100,19 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
             Qres <- lagDistrImpacts(T, g, P, q=as.integer(Q))
             attr(res, "Qres") <- Qres
         }
+        timings[["trace_impacts"]] <- proc.time() - .ptime_start
+        .ptime_start <- proc.time()
         if (!is.null(R)) {
             samples <- mvrnorm(n=R, mu=mu, Sigma=Sigma, tol=tol,
                 empirical=empirical)
+            if (mess) samples[,irho] <- 1 - exp(samples[,irho])
             if (!is.null(interval)) {
                 check <- ((samples[,irho] > interval[1]) & 
                     (samples[,irho] < interval[2]))
                 if (any(!check)) samples <- samples[check,]
             }
+            timings[["impacts_samples"]] <- proc.time() - .ptime_start
+            .ptime_start <- proc.time()
             processSample <- function(x, irho, drop2beta) {
                 g <- x[irho]^(0:q)
                 beta <- x[-drop2beta]
@@ -126,8 +139,38 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
                 }
                 res
             }
-            sres <- apply(samples, 1, processSample, irho=irho,
-                drop2beta=drop2beta)
+            CL <- get("cl", env = .spdepOptions)
+            if (!is.null(CL) && length(CL) > 1) {
+                require(snow)
+                l_sp <- lapply(splitIndices(nrow(samples), length(CL)), 
+		    function(i) samples[i,])
+		clusterExport_l <- function(CL, list) {
+                    gets <- function(n, v) {
+                        assign(n, v, env = .GlobalEnv)
+                        NULL
+                    }
+                    for (name in list) {
+                        clusterCall(CL, gets, name, get(name))
+                    }
+		}
+		clusterExport_l(CL, list("processSample", "irho", "drop2beta",
+                    "Q", "T", "lagImpacts", "lagDistrImpacts", "icept",
+                    "iicept", "type"))
+                timings[["cluster_setup"]] <- proc.time() - .ptime_start
+                .ptime_start <- proc.time()
+                lsres <- parLapply(CL, l_sp, function(sp) apply(sp, 1, 
+                    processSample, irho=irho, drop2beta=drop2beta))
+		clusterEvalQ(CL, rm(list=c("processSample", "irho", "drop2beta",
+                    "Q", "T", "lagImpacts", "lagDistrImpacts", "icept",
+                    "iicept", "type")))
+                sres <- do.call("c", lsres)
+
+            } else {
+                sres <- apply(samples, 1, processSample, irho=irho,
+                    drop2beta=drop2beta)
+            }
+            timings[["process_samples"]] <- proc.time() - .ptime_start
+            .ptime_start <- proc.time()
             direct <- as.mcmc(t(sapply(sres, function(x) x$direct)))
             indirect <- as.mcmc(t(sapply(sres, function(x) x$indirect)))
             total <- as.mcmc(t(sapply(sres, function(x) x$total)))
@@ -150,6 +193,7 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
                 Qmcmc <- list(direct=Qdirect, indirect=Qindirect, total=Qtotal)
                 attr(ssres, "Qmcmc") <- Qmcmc
             }
+            timings[["postprocess_samples"]] <- proc.time() - .ptime_start
             res <- list(res=res, sres=ssres)
         }
         attr(res, "method") <- "trace"
@@ -177,6 +221,8 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
         SW <- invIrW(listw, rho)
         if (type == "lag") res <- lagImpactsExact(SW, P, n)
         else if (type == "mixed") res <- mixedImpactsExact(SW, P, n, listw)
+        timings[["weights_impacts"]] <- proc.time() - .ptime_start
+        .ptime_start <- proc.time()
         if (!is.null(R)) {
             samples <- mvrnorm(n=R, mu=mu, Sigma=Sigma, tol=tol,
                 empirical=empirical)
@@ -185,6 +231,8 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
                     (samples[,irho] < interval[2]))
                 if (any(!check)) samples <- samples[check,]
             }
+            timings[["impacts_samples"]] <- proc.time() - .ptime_start
+            .ptime_start <- proc.time()
             processXSample <- function(x, drop2beta) {
                 beta <- x[-drop2beta]
                 if (type == "lag") {
@@ -204,13 +252,47 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
                     return(mixedImpactsExact(SW, P, n, listw))
                 }
             }
-            sres <- apply(samples, 1, processXSample, drop2beta=drop2beta)
+            CL <- get("cl", env = .spdepOptions)
+            if (!is.null(CL) && length(CL) > 1) {
+                require(snow)
+                l_sp <- lapply(splitIndices(nrow(samples), length(CL)), 
+		    function(i) samples[i,])
+		clusterExport_l <- function(CL, list) {
+                    gets <- function(n, v) {
+                        assign(n, v, env = .GlobalEnv)
+                        NULL
+                    }
+                    for (name in list) {
+                        clusterCall(CL, gets, name, get(name))
+                    }
+		}
+
+		clusterExport_l(CL, list("processXSample", "irho", "drop2beta",
+                    "SW", "lagImpactsExact", "mixedImpactsExact", "icept",
+                    "iicept", "type", "listw"))
+
+                timings[["cluster_setup"]] <- proc.time() - .ptime_start
+                .ptime_start <- proc.time()
+                lsres <- parLapply(CL, l_sp, function(sp) apply(sp, 1, 
+                    processXSample, drop2beta=drop2beta))
+		clusterEvalQ(CL, rm(list=c("processXSample", "drop2beta",
+                    "SW", "lagImpactsExact", "mixedImpactsExact", "icept",
+                    "iicept", "type", "listw")))
+                sres <- do.call("c", lsres)
+
+            } else {
+                sres <- apply(samples, 1, processXSample,
+                    drop2beta=drop2beta)
+            }
+            timings[["process_samples"]] <- proc.time() - .ptime_start
+            .ptime_start <- proc.time()
             direct <- as.mcmc(t(sapply(sres, function(x) x$direct)))
             indirect <- as.mcmc(t(sapply(sres, function(x) x$indirect)))
             total <- as.mcmc(t(sapply(sres, function(x) x$total)))
             colnames(direct) <- bnames
             colnames(indirect) <- bnames
             colnames(total) <- bnames
+            timings[["postprocess_samples"]] <- proc.time() - .ptime_start
             res <- list(res=res, sres=list(direct=direct,
                 indirect=indirect, total=total))
         }
@@ -219,10 +301,17 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
     attr(res, "type") <- type
     attr(res, "bnames") <- bnames
     attr(res, "haveQ") <- !is.null(Q)
+    attr(res, "timings") <- do.call("rbind", timings)[, c(1,3)]
     class(res) <- "lagImpact"
     res
 }
 
+impacts.lagmess <- function(obj, ..., tr=NULL, R=NULL, listw=NULL, 
+  tol=1e-6, empirical=FALSE, Q=NULL) {
+    stopifnot(!is.null(obj$mixedHess))
+    stop("method not yet available")
+}
+
 impacts.sarlm <- function(obj, ..., tr=NULL, R=NULL, listw=NULL, useHESS=NULL,
   tol=1e-6, empirical=FALSE, Q=NULL) {
     if (obj$type == "error")
