@@ -14,9 +14,12 @@ joincount <- function(dums, listw) {
 	res
 }
 
-joincount.test <- function(fx, listw, zero.policy=FALSE,
+joincount.test <- function(fx, listw, zero.policy=NULL,
 	alternative="greater", #adjust.n=TRUE, 
 	spChk=NULL, adjust.n=TRUE) {
+        if (is.null(zero.policy))
+            zero.policy <- get("zeroPolicy", env = .spdepOptions)
+        stopifnot(is.logical(zero.policy))
 	alternative <- match.arg(alternative, c("greater", "less", "two.sided"))
 	if (!inherits(listw, "listw")) stop(paste(deparse(substitute(listw)),
 		"is not a listw object"))
