@@ -1,9 +1,12 @@
-# Copyright 2001-8 by Roger S. Bivand and Virgilio Gomez-Rubio
+# Copyright 2001-10 by Roger S. Bivand and Virgilio Gomez-Rubio
 #
 
-nb2listw <- function(neighbours, glist=NULL, style="W", zero.policy=FALSE)
+nb2listw <- function(neighbours, glist=NULL, style="W", zero.policy=NULL)
 {
 	if(!inherits(neighbours, "nb")) stop("Not a neighbours list")
+        if (is.null(zero.policy))
+            zero.policy <- get("zeroPolicy", env = .spdepOptions)
+        stopifnot(is.logical(zero.policy))
 	if (!(style %in% c("W", "B", "C", "S", "U")))
 		stop(paste("Style", style, "invalid"))
 	n <- length(neighbours)
