@@ -1,8 +1,11 @@
-# Copyright 2002 by Roger Bivand 
+# Copyright 2002-2010 by Roger Bivand 
 #
 
-sp.mantel.mc <- function(var, listw, nsim, type="moran", zero.policy=FALSE,
-	alternative="greater", spChk=NULL) {
+sp.mantel.mc <- function(var, listw, nsim, type="moran", zero.policy=NULL,
+	alternative="greater", spChk=NULL, return_boot=FALSE) {
+        if (is.null(zero.policy))
+            zero.policy <- get("zeroPolicy", env = .spdepOptions)
+        stopifnot(is.logical(zero.policy))
 	alternative <- match.arg(alternative, c("greater", "less"))
 	if(!inherits(listw, "listw")) stop(paste(deparse(substitute(listw)),
 		"is not a listw object"))
@@ -44,6 +47,26 @@ sp.mantel.mc <- function(var, listw, nsim, type="moran", zero.policy=FALSE,
 	else if (type == "sokal") f <- mantel.sokal
 	else stop("unknown type")
 	xs <- scale(var)
+        if (return_boot) {
+            mantel_boot <- function(var, i, ...) {
+                var <- var[i]
+                return(f(x=var, ...))
+            }
+            cl <- get("cl", env = .spdepOptions)
+            if (!is.null(cl) && length(cl) > 1) {
+                nnsim <- boot_wrapper_in(cl, nsim)
+                lres <- clusterCall(cl, boot, xs, statistic=mantel_boot,
+                    R=nnsim, sim="permutation", listwU=listw.U,
+                    zero.policy=zero.policy)
+                res <- boot_wrapper_out(lres, match.call())
+            } else {
+                res <- boot(xs, statistic=mantel_boot, R=nsim,
+                    sim="permutation", listwU=listw.U, 
+                    zero.policy=zero.policy)
+            }
+            return(res)
+        }
+
 	res <- numeric(length=nsim+1)
 	for (i in 1:nsim) {
 		y <- sample(xs)
